-- scope stuff
qualityoflife = qualityoflife or {};
local q = qualityoflife;

q.modules = q.modules or {};
q.modules["QoL_ClassicHeals"] = q.modules["QoL_ClassicHeals"] or {};
local c = q.modules["QoL_ClassicHeals"];

function c:Println(str)
	if str then
		q:Println(str, "ClassicHeals");
	end
end

function c:GetAffectedPartyMemberId(memberName)
	for playerId, playerName in ipairs(q:GetPartyMembers()) do
		if playerName == memberName then
			return playerId;
		end
	end
end

function c:GetAffectedRaidMemberId(memberName)
	local rm = q:GetRaidMembers();
	local raidId;

	for playerId, playerName in ipairs(rm[0]) do
		if playerName == memberName then
			raidId = playerId;
			break;
		end
	end

	for subGroupId, subGroup in pairs(rm) do
		if subGroupId > 0 then
			for playerInSubGroupId, playerName in ipairs(rm[subGroupId]) do
				if playerName == memberName then
					return raidId, subGroupId, playerInSubGroupId;
				end;
			end;
		end
	end

	if memberName == GetUnitName("player") then
		return 1, 1, 1; -- for grid, vuhdo etc.
	end
end

function c:GetCompactRaidFrameId(memberName)
	for n = 1, 40 do
		local compactRaidFrameName = _G["CompactRaidFrame" .. n .. "Name"];
		if compactRaidFrameName and compactRaidFrameName:GetText() == memberName then
			return n;
		end
	end
end

function c:GetRaidPulloutId(subGroupId)
	for n = 1, 8 do
		local raidPulloutName = _G["RaidPullout" .. n .. "Name"];
		if raidPulloutName then
			local pulloutText = raidPulloutName:GetText();
			local textLen = string.len(pulloutText);
			local pulloutGroupId = tonumber(string.sub(pulloutText, textLen, textLen + 1));
			if pulloutGroupId == subGroupId then
				return n;
			end
		end
	end
end

function c:GetTargetFrames(target)
	local result = {};

	----- default ui -----

	-- player frames
	if QOL_OPTIONS[c.name][c.PROFILE_PLAYER][c.OPT_UIATTR_ENABLED] and target == GetUnitName("player") then
		result[c.PROFILE_PLAYER] = {};

		-- default player frame
		if _G["PlayerFrame"] and _G["PlayerFrame"]:IsVisible() then
			table.insert(result[c.PROFILE_PLAYER], _G["PlayerFrame"]);
		end

		-- elvui player frame
		if _G["ElvUF_Player"] and _G["ElvUF_Player"]:IsVisible() then
			table.insert(result[c.PROFILE_PLAYER], _G["ElvUF_Player"]);
		end
	end

	-- target frames
	if QOL_OPTIONS[c.name][c.PROFILE_TARGET][c.OPT_UIATTR_ENABLED] and target == GetUnitName("target") then
		result[c.PROFILE_TARGET] = {};

		-- default target frame
		if _G["TargetFrame"] and _G["TargetFrame"]:IsVisible() then
			table.insert(result[c.PROFILE_TARGET], _G["TargetFrame"]);
		end

		-- elvui target frame
		if _G["ElvUF_Target"] and _G["ElvUF_Target"]:IsVisible() then
			table.insert(result[c.PROFILE_TARGET], _G["ElvUF_Target"]);
		end
	end

	-- party frames
	if QOL_OPTIONS[c.name][c.PROFILE_PARTY][c.OPT_UIATTR_ENABLED] then
		local partyMemberId = c:GetAffectedPartyMemberId(target);
		if partyMemberId then
			result[c.PROFILE_PARTY] = {};

			-- default party frame
			local defaultPartyFrameName = "PartyMemberFrame" .. partyMemberId;
			if _G[defaultPartyFrameName] and _G[defaultPartyFrameName]:IsVisible() then
				table.insert(result[c.PROFILE_PARTY], _G[defaultPartyFrameName]);
			end

			-- default party frame
			local elvUiPartyFrameName = "ElvUF_PartyGroup1UnitButton" .. partyMemberId + 1;
			if _G[elvUiPartyFrameName] and _G[elvUiPartyFrameName]:IsVisible() then
				table.insert(result[c.PROFILE_PARTY], _G[elvUiPartyFrameName]);
			end
		end
	end

	-- raid frames
	local raidId, subGroupId, playerInSubGroupId = c:GetAffectedRaidMemberId(target);

	if raidId and subGroupId and playerInSubGroupId and (QOL_OPTIONS[c.name][c.PROFILE_RAID_COMPACT][c.OPT_UIATTR_ENABLED] or QOL_OPTIONS[c.name][c.PROFILE_RAID_PULLOUT][c.OPT_UIATTR_ENABLED]) then
		if QOL_OPTIONS[c.name][c.PROFILE_RAID_COMPACT][c.OPT_UIATTR_ENABLED] then
			result[c.PROFILE_RAID_COMPACT] = {};

			-- default compactraidgroup / compactraidframe
			if _G["CompactRaidFrameContainer"]:IsVisible() then
				local compactRaidFrameName;
				if CompactRaidFrameManager_GetSetting("KeepGroupsTogether") then
					compactRaidFrameName = "CompactRaidGroup" .. subGroupId .. "Member" .. playerInSubGroupId
				else
					local compactRaidFrameId = c:GetCompactRaidFrameId(target);
					if compactRaidFrameId then
						compactRaidFrameName = "CompactRaidFrame" .. compactRaidFrameId;
					end
				end

				if compactRaidFrameName and _G[compactRaidFrameName] and _G[compactRaidFrameName]:IsVisible() then
					table.insert(result[c.PROFILE_RAID_COMPACT], _G[compactRaidFrameName]);
				end
			end
		end

		if QOL_OPTIONS[c.name][c.PROFILE_RAID_PULLOUT][c.OPT_UIATTR_ENABLED] then
			result[c.PROFILE_RAID_PULLOUT] = {};

			-- default raid pull out group
			local pulloutGroupId = c:GetRaidPulloutId(subGroupId);
			if pulloutGroupId then
				local pulloutRaidFrameName = "RaidPullout" .. pulloutGroupId .. "Button" .. playerInSubGroupId;
				if _G[pulloutRaidFrameName] and _G[pulloutRaidFrameName]:IsVisible() then						
					table.insert(result[c.PROFILE_RAID_PULLOUT], _G[pulloutRaidFrameName]);
				end
			end
		end
	end

	----- addons -----

	-- grid2
	if subGroupId and playerInSubGroupId and QOL_OPTIONS[c.name][c.PROFILE_ADDON_GRID2][c.OPT_UIATTR_ENABLED] then
		result[c.PROFILE_ADDON_GRID2] = {};

		local gridFrameName = "Grid2LayoutHeader" .. subGroupId .. "UnitButton" .. playerInSubGroupId;
		if _G[gridFrameName] and _G[gridFrameName]:IsVisible() and _G[gridFrameName].health then
			table.insert(result[c.PROFILE_ADDON_GRID2], _G[gridFrameName].health);
		end
	end

	-- vuhdo
	if QOL_OPTIONS[c.name][c.PROFILE_ADDON_VUHDO][c.OPT_UIATTR_ENABLED] then
		result[c.PROFILE_ADDON_VUHDO] = {};

		-- target
		if UnitName("target") then
			local vuhdoTargetFrameName = c:GetVuhdoFrame("target");
			if vuhdoTargetFrameName and _G[vuhdoTargetFrameName .. "BgBarIcBarHlBarIc"] and _G[vuhdoTargetFrameName .. "BgBarIcBarHlBarIc"]:IsVisible() then
				table.insert(result[c.PROFILE_ADDON_VUHDO], _G[vuhdoTargetFrameName .. "BgBarIcBarHlBarIc"]);
			end
		end

		-- player/raid
		local vuhdoRaidFrameName = c:GetVuhdoFrame(target);
		if vuhdoRaidFrameName and _G[vuhdoRaidFrameName .. "BgBarIcBarHlBarIc"] and _G[vuhdoRaidFrameName .. "BgBarIcBarHlBarIc"]:IsVisible() then
			table.insert(result[c.PROFILE_ADDON_VUHDO], _G[vuhdoRaidFrameName .. "BgBarIcBarHlBarIc"]);
		end
	end

	return result;
end

function c:GetVuhdoFrame(target)
	if VUHDO_PANEL_UNITS then -- a vuhdo global variable. if exist -> addon installed
		if target == GetUnitName("player") then
			target = "player";
		else
			local partyMemberId, raidMemberId = c:GetAffectedPartyMemberId(target), c:GetAffectedRaidMemberId(target);
			if IsInRaid() and raidMemberId then
				target = "raid" .. raidMemberId;
			elseif IsInGroup() and partyMemberId then
				target = "party" .. partyMemberId;
			end
		end

		-- kind of a brute force attempt. but the vuhdo code is way too obfuscated ^^
		for groupId = 1, 8 do
			for playerId = 1, 5 do
				local frameName = "Vd" .. groupId .. "H" .. playerId;
				if _G[frameName] and _G[frameName].raidid and _G[frameName].raidid == target then
					return frameName;
				end
			end
		end
	end
end

function c:ShowResetDialog()
	local dialogName = c.name .. "_RESETDIALOG";
	StaticPopupDialogs[dialogName] = StaticPopupDialogs[dialogName] or {
		text = q:GetText("This will reset all ClassicHeals settings to defaults.\n\nAre you sure?"),
		button1 = q:GetText("Reset"),
		button2 = q:GetText("Cancel"),
		OnEscapePressed = function ()
			self:Hide();
		end,
		OnAccept = function()
			QOL_OPTIONS[c.name] = {};
			c:InitOptions();
			c:SaveSettingsChanges();
			c:SetProfileValues();
		end,
		timeout = 0,
		whileDead = true,
		hideOnEscape = true,
		preferredIndex = 3
	};
	StaticPopup_Show(dialogName);
end